@@ -145,11 +145,6 @@ DefParam("preferlists",
 	"b",
 	1);
 
-DefParam("prettyasciimail",
-	"If this is on, Bugzilla will send email reports formatted (assuming 76 character monospace font display). If this is off, email reports are sent using the old 'one-item-per-line' format.",
-	"b",
-	0);
-
 DefParam("capitalizelists",
 	"If this is on, Bugzilla will capitalize list entries, checkboxes, and radio buttons. If this is off, Bugzilla will leave these items untouched.",
 	"b",
@@ -374,56 +369,33 @@ To use the wonders of Bugzilla, you can use the following:
 });
 
 
-
-DefParam("changedmail",
-q{The email that gets sent to people when a bug changes.  Within this
-text, %to% gets replaced by the assigned-to and reported-by people,
-separated by a comma (with duplication removed, if they're the same
-person).  %cc% gets replaced by the list of people on the CC list,
-separated by commas.  %bugid% gets replaced by the bug number.
-%diffs% gets replaced by the diff text from the old version to the new
-version of this bug.  %neworchanged% is "New: " if this mail is
-reporting a new bug or empty if changes were made
-to an existing one.  %summary% gets replaced by the summary of this
-bug.  %<i>anythingelse</i>% gets replaced by the definition of that
-parameter (as defined on this page).},
-         "l",
-"From: bugzilla-daemon
-To: %to%
-Cc: %cc%
-Subject: [Bug %bugid%] %neworchanged%%summary%
-
-%urlbase%show_bug.cgi?id=%bugid%
-
-%diffs%");
-
-DefParam("newemailtech",
-q{The way that email diffs are constructed by Bugzilla.  You can revert to
-the old technology by turning this off, but this is not advised, as the old 
-email tech will probably disappear in the next version of Bugzilla.  Some
-features (watches, server side mail filtering) depend on newemailtech being
-set to on.},
-    "b",
-    1);
-
-DefParam("newemailtechdefault",
-q{Make "newemailtech" the default for all new accounts created.  This will
-not change any existing accounts nor will it remove a users ability to go
-back to the oldmail system (requires "newemailtech" to be on as well).},
-    "b",
-    1);
-
 DefParam("newchangedmail",
-q{The same as 'changedmail', but used for the newemailtech stuff.},
+q{The email that gets sent to people when a bug changes.  Within this
+text, %to% gets replaced with the e-mail address of the person recieving
+the mail.  %bugid% gets replaced by the bug number.  %diffs% gets
+replaced with what's changed.  %neworchanged% is "New:" if this mail is
+reporting a new bug or empty if changes were made to an existing one.
+%summary% gets replaced by the summary of this bug. %reasonsheader% 
+is replaced by an abbreviated list of reasons why the user is getting the email,
+suitable for use in an email header (such as X-Bugzilla-Reason).
+%reasonsbody% is replaced by text that explains why the user is getting the email 
+in more user friendly text than %reasonsheader%.
+%<i>anythingelse</i>% gets replaced by the definition of 
+that parameter (as defined on this
+page).},
          "l",
 "From: bugzilla-daemon
 To: %to%
-Cc: %cc%
 Subject: [Bug %bugid%] %neworchanged%%summary%
+X-Bugzilla-Reason: %reasonsheader%
 
 %urlbase%show_bug.cgi?id=%bugid%
 
-%diffs%");
+%diffs%
+
+
+
+%reasonsbody%");
 
 
 
@@ -567,14 +539,14 @@ DefParam("expectbigqueries",
          0);
 
 DefParam("emailregexp",
-         'This defines the regexp to use for legal email addresses.  The default tries to match fully qualified email addresses.  Another popular value to put here is <tt>^[^@, ]*$</tt>, which means "local usernames, no @ allowed.',
+         'This defines the regexp to use for legal email addresses.  The default tries to match fully qualified email addresses.  Another popular value to put here is <tt>^[^@]*$</tt>, which means "local usernames, no @ allowed.',
          "t",
-         q:^[^@, ]*@[^@, ]*\\.[^@, ]*$:);
+         q:^[^@]*@[^@]*\\.[^@]*$:);
 
 DefParam("emailregexpdesc",
          "This describes in english words what kinds of legal addresses are allowed by the <tt>emailregexp</tt> param.",
          "l",
-         "A legal address must contain exactly one '\@', and at least one '.' after the \@, and may not contain any commas or spaces.");
+         "A legal address must contain exactly one '\@', and at least one '.' after the \@.");
 
 DefParam("emailsuffix",
          "This is a string to append to any email addresses when actually sending mail to that address.  It is useful if you have changed the <tt>emailregexp</tt> param to only allow local usernames, but you want the mail to be delivered to username\@my.local.hostname.",
@@ -583,18 +555,20 @@ DefParam("emailsuffix",
 
 
 DefParam("voteremovedmail",
-q{This is a mail message to send to anyone who gets a vote removed from a bug for any reason.  %to% gets replaced by a comma-separated list of people who used to be voting for this bug.  %bugid% gets replaced by the bug number.  %reason% gets replaced by a short reason describing why the vote was removed.  %count% is how many votes got removed.%<i>anythingelse</i>% gets replaced by the definition of that parameter (as defined on this page).},
+q{This is a mail message to send to anyone who gets a vote removed from a bug for any reason.  %to% gets replaced by the person who used to be voting for this bug.  %bugid% gets replaced by the bug number.  %reason% gets replaced by a short reason describing why the vote(s) were removed.  %votesremoved%, %votesold% and %votesnew% is the number of votes removed, before and after respectively.  %votesremovedtext%, %votesoldtext% and %votesnewtext% are these as sentences, eg "You had 2 votes on this bug."  %count% is also supported for backwards compatibility.  %<i>anythingelse</i>% gets replaced by the definition of that parameter (as defined on this page).},
          "l",
 "From: bugzilla-daemon
 To: %to%
-Subject: [Bug %bugid%] Your vote has been removed from this bug
+Subject: [Bug %bugid%] Some or all of your votes have been removed.
 
-You used to have a vote on bug %bugid%, but it has been removed.
+Some or all of your votes have been removed from bug %bugid%.
+
+%votesoldtext%
+
+%votesnewtext%
 
 Reason: %reason%
 
-Votes removed: %count%
-
 %urlbase%show_bug.cgi?id=%bugid%
 ");
          
