@@ -31,6 +31,7 @@ use strict;
 
 sub globals_pl_sillyness {
     my $zz;
+    $zz = @main::SqlStateStack;
     $zz = @main::chooseone;
     $zz = @main::default_column_list;
     $zz = $main::defaultqueryname;
@@ -48,6 +49,7 @@ sub globals_pl_sillyness {
     $zz = @main::legal_versions;
     $zz = @main::milestoneurl;
     $zz = @main::prodmaxvotes;
+    $zz = $main::superusergroupset;
 }
 
 #
@@ -66,11 +68,14 @@ use DBI;
 
 use Date::Format;               # For time2str().
 use Date::Parse;               # For str2time().
-# use Carp;                       # for confess
+#use Carp;                       # for confess
 use RelationSet;
 
+# $ENV{PATH} is not taint safe
+delete $ENV{PATH};
+
 # Contains the version string for the current running Bugzilla.
-$::param{'version'} = '2.12';
+$::param{'version'} = '2.14';
 
 $::dontchange = "--do_not_change--";
 $::chooseone = "--Choose_one:--";
@@ -82,6 +87,13 @@ $::dbwritesallowed = 1;
 # Joe Robins, 7/5/00
 $::superusergroupset = "9223372036854775807";
 
+#sub die_with_dignity {
+#    my ($err_msg) = @_;
+#    print $err_msg;
+#    confess($err_msg);
+#}
+#$::SIG{__DIE__} = \&die_with_dignity;
+
 sub ConnectToDatabase {
     my ($useshadow) = (@_);
     if (!defined $::db) {
@@ -107,8 +119,28 @@ sub ReconnectToShadowDatabase {
 my $shadowchanges = 0;
 sub SyncAnyPendingShadowChanges {
     if ($shadowchanges) {
-        system("./syncshadowdb &");
-        $shadowchanges = 0;
+        my $pid;
+        FORK: {
+            if ($pid = fork) { # create a fork
+                # parent code runs here
+                $shadowchanges = 0;
+                return;
+            } elsif (defined $pid) {
+                # child process code runs here
+                exec("./syncshadowdb","--") or die "Unable to exec syncshadowdb: $!";
+                # the idea was that passing the second parameter tricks it into
+                # using execvp instead of running a shell. Not really necessary since
+                # there are no shell meta-characters, but it passes our tinderbox
+                # test that way. :) http://bugzilla.mozilla.org/show_bug.cgi?id=21253
+            } elsif ($! =~ /No more process/) {
+                # recoverable fork error, try again in 5 seconds
+                sleep 5;
+                redo FORK;
+            } else {
+                # something weird went wrong
+                die "Can't create background process to run syncshadowdb: $!";
+            }
+        }
     }
 }
 
@@ -168,6 +200,15 @@ sub SendSQL {
     if ($str =~ /^LOCK TABLES/i && $str !~ /shadowlog/ && $::dbwritesallowed) {
         $str =~ s/^LOCK TABLES/LOCK TABLES shadowlog WRITE, /i;
     }
+    # If we are shutdown, we don't want to run queries except in special cases
+    if (Param('shutdownhtml')) {
+        if ($0 =~ m:[\\/]((do)?editparams.cgi|syncshadowdb)$:) {
+            $::ignorequery = 0;
+        } else {
+            $::ignorequery = 1;
+            return;
+        }
+    }
     SqlLog($str);
     $::currentquery = $::db->prepare($str);
     $::currentquery->execute
@@ -189,6 +230,10 @@ sub SendSQL {
 }
 
 sub MoreSQLData {
+    # $::ignorequery is set in SendSQL
+    if ($::ignorequery) {
+        return 0;
+    }
     if (defined @::fetchahead) {
 	return 1;
     }
@@ -199,6 +244,10 @@ sub MoreSQLData {
 }
 
 sub FetchSQLData {
+    # $::ignorequery is set in SendSQL
+    if ($::ignorequery) {
+        return;
+    }
     if (defined @::fetchahead) {
 	my @result = @::fetchahead;
 	undef @::fetchahead;
@@ -479,7 +528,7 @@ sub GenerateVersionTable {
 
     foreach my $i (@list) {
         if (!defined $::components{$i}) {
-            $::components{$i} = "";
+            $::components{$i} = [];
         }
     }
     @::legal_versions = sort {uc($a) cmp uc($b)} keys(%varray);
@@ -559,7 +608,9 @@ sub ModTime {
 
 # This proc must be called before using legal_product or the versions array.
 
+$::VersionTableLoaded = 0;
 sub GetVersionTable {
+    return if $::VersionTableLoaded;
     my $mtime = ModTime("data/versioncache");
     if (!defined $mtime || $mtime eq "") {
         $mtime = 0;
@@ -576,25 +627,27 @@ sub GetVersionTable {
             die "Can't generate file data/versioncache";
         }
     }
+    $::VersionTableLoaded = 1;
 }
 
 
 sub InsertNewUser {
     my ($username, $realname) = (@_);
-    my $password = "";
-    for (my $i=0 ; $i<8 ; $i++) {
-        $password .= substr("abcdefghijklmnopqrstuvwxyz", int(rand(26)), 1);
-    }
-    my $usenewemailtech = Param('newemailtech') & Param('newemailtechdefault');
 
+    # Generate a new random password for the user.
+    my $password = GenerateRandomPassword();
+    my $cryptpassword = Crypt($password);
+
+    # Determine what groups the user should be in by default
+    # and add them to those groups.
     PushGlobalSQLState();
     SendSQL("select bit, userregexp from groups where userregexp != ''");
     my $groupset = "0";
     while (MoreSQLData()) {
         my @row = FetchSQLData();
-	# Modified -Joe Robins, 2/17/00
-	# Making this case insensitive, since usernames are email addresses,
-	# and could be any case.
+        # Modified -Joe Robins, 2/17/00
+        # Making this case insensitive, since usernames are email addresses,
+        # and could be any case.
         if ($username =~ m/$row[1]/i) {
             $groupset .= "+ $row[0]"; # Silly hack to let MySQL do the math,
                                       # not Perl, since we're dealing with 64
@@ -602,14 +655,100 @@ sub InsertNewUser {
                                       # does that.
         }
     }
-            
+
+    # Insert the new user record into the database.            
     $username = SqlQuote($username);
     $realname = SqlQuote($realname);
-    SendSQL("insert into profiles (login_name, realname, password, cryptpassword, groupset, newemailtech) values ($username, $realname, '$password', encrypt('$password'), $groupset, $usenewemailtech)");
+    $cryptpassword = SqlQuote($cryptpassword);
+    SendSQL("INSERT INTO profiles (login_name, realname, cryptpassword, groupset) 
+             VALUES ($username, $realname, $cryptpassword, $groupset)");
     PopGlobalSQLState();
+
+    # Return the password to the calling code so it can be included 
+    # in an email sent to the user.
     return $password;
 }
 
+sub GenerateRandomPassword {
+    my ($size) = @_;
+
+    # Generated passwords are eight characters long by default.
+    $size ||= 8;
+
+    # The list of characters that can appear in a randomly generated password.
+    # Note that users can put any character into a password they choose themselves.
+    my @pwchars = (0..9, 'A'..'Z', 'a'..'z', '-', '_', '!', '@', '#', '$', '%', '^', '&', '*');
+
+    # The number of characters in the list.
+    my $pwcharslen = scalar(@pwchars);
+
+    # Generate the password.
+    my $password = "";
+    for ( my $i=0 ; $i<$size ; $i++ ) {
+        $password .= $pwchars[rand($pwcharslen)];
+    }
+
+    # Return the password.
+    return $password;
+}
+
+
+sub ValidatePassword {
+    # Determines whether or not a password is valid (i.e. meets Bugzilla's
+    # requirements for length and content).  If the password is valid, the
+    # function returns boolean false.  Otherwise it returns an error message
+    # (synonymous with boolean true) that can be displayed to the user.
+    
+    # If a second password is passed in, this function also verifies that
+    # the two passwords match.
+
+    my ($password, $matchpassword) = @_;
+    
+    if ( length($password) < 3 ) {
+        return "The password is less than three characters long.  It must be at least three characters.";
+    } elsif ( length($password) > 16 ) {
+        return "The password is more than 16 characters long.  It must be no more than 16 characters.";
+    } elsif ( $matchpassword && $password ne $matchpassword ) { 
+        return "The two passwords do not match.";
+    }
+
+    return 0;
+}
+
+
+sub Crypt {
+    # Crypts a password, generating a random salt to do it.
+    # Random salts are generated because the alternative is usually
+    # to use the first two characters of the password itself, and since
+    # the salt appears in plaintext at the beginning of the crypted
+    # password string this has the effect of revealing the first two
+    # characters of the password to anyone who views the crypted version.
+
+    my ($password) = @_;
+
+    # The list of characters that can appear in a salt.  Salts and hashes
+    # are both encoded as a sequence of characters from a set containing
+    # 64 characters, each one of which represents 6 bits of the salt/hash.
+    # The encoding is similar to BASE64, the difference being that the
+    # BASE64 plus sign (+) is replaced with a forward slash (/).
+    my @saltchars = (0..9, 'A'..'Z', 'a'..'z', '.', '/');
+
+    # Generate the salt.  We use an 8 character (48 bit) salt for maximum
+    # security on systems whose crypt uses MD5.  Systems with older
+    # versions of crypt will just use the first two characters of the salt.
+    my $salt = '';
+    for ( my $i=0 ; $i < 8 ; ++$i ) {
+        $salt .= $saltchars[rand(64)];
+    }
+
+    # Crypt the password.
+    my $cryptedpassword = crypt($password, $salt);
+
+    # Return the crypted password.
+    return $cryptedpassword;
+}
+
+
 sub DBID_to_real_or_loginname {
     my ($id) = (@_);
     PushGlobalSQLState();
@@ -625,6 +764,12 @@ sub DBID_to_real_or_loginname {
 
 sub DBID_to_name {
     my ($id) = (@_);
+    # $id should always be a positive integer
+    if ($id =~ m/^([1-9][0-9]*)$/) {
+        $id = $1;
+    } else {
+        $::cachedNameArray{$id} = "__UNKNOWN__";
+    }
     if (!defined $::cachedNameArray{$id}) {
         PushGlobalSQLState();
         SendSQL("select login_name from profiles where userid = $id");
@@ -644,15 +789,18 @@ sub DBname_to_id {
     SendSQL("select userid from profiles where login_name = @{[SqlQuote($name)]}");
     my $r = FetchOneColumn();
     PopGlobalSQLState();
-    if (!defined $r || $r eq "") {
+    # $r should be a positive integer, this makes Taint mode happy
+    if (defined $r && $r =~ m/^([1-9][0-9]*)$/) {
+        return $1;
+    } else {
         return 0;
     }
-    return $r;
 }
 
 
 sub DBNameToIdAndCheck {
     my ($name, $forceok) = (@_);
+    $name = html_quote($name);
     my $result = DBname_to_id($name);
     if ($result > 0) {
         return $result;
@@ -666,6 +814,7 @@ sub DBNameToIdAndCheck {
         print "Yikes; couldn't create user $name.  Please report problem to " .
             Param("maintainer") ."\n";
     } else {
+        print "\n";  # http://bugzilla.mozilla.org/show_bug.cgi?id=80045
         print "The name <TT>$name</TT> is not a valid username.  Either you\n";
         print "misspelled it, or the person has not registered for a\n";
         print "Bugzilla account.\n";
@@ -674,6 +823,18 @@ sub DBNameToIdAndCheck {
     exit(0);
 }
 
+# Use detaint_string() when you know that there is no way that the data
+# in a scalar can be tainted, but taint mode still bails on it.
+# WARNING!! Using this routine on data that really could be tainted
+#           defeats the purpose of taint mode.  It should only be
+#           used on variables that cannot be touched by users.
+
+sub detaint_string {
+    my ($str) = @_;
+    $str =~ m/^(.*)$/s;
+    $str = $1;
+}
+
 # This routine quoteUrls contains inspirations from the HTML::FromText CPAN
 # module by Gareth Rees <garethr@cre.canon.co.uk>.  It has been heavily hacked,
 # all that is really recognizable from the original is bits of the regular
@@ -940,6 +1101,8 @@ sub SqlQuote {
 #     }
     $str =~ s/([\\\'])/\\$1/g;
     $str =~ s/\0/\\0/g;
+    # If it's been SqlQuote()ed, then it's safe, so we tell -T that.
+    $str = detaint_string($str);
     return "'$str'";
 }
 
@@ -959,6 +1122,16 @@ sub UserInGroup {
     return 0;
 }
 
+sub BugInGroup {
+    my ($bugid, $groupname) = (@_);
+    my $groupbit = GroupNameToBit($groupname);
+    PushGlobalSQLState();
+    SendSQL("SELECT (bugs.groupset & $groupbit) != 0 FROM bugs WHERE bugs.bug_id = $bugid");
+    my $bugingroup = FetchOneColumn();
+    PopGlobalSQLState();
+    return $bugingroup;
+}
+
 sub GroupExists {
     my ($groupname) = (@_);
     ConnectToDatabase();
@@ -967,6 +1140,31 @@ sub GroupExists {
     return $count;
 }
 
+# Given the name of an existing group, returns the bit associated with it.
+# If the group does not exist, returns 0.
+# !!! Remove this function when the new group system is implemented!
+sub GroupNameToBit {
+    my ($groupname) = (@_);
+    ConnectToDatabase();
+    PushGlobalSQLState();
+    SendSQL("SELECT bit FROM groups WHERE name = " . SqlQuote($groupname));
+    my $bit = FetchOneColumn() || 0;
+    PopGlobalSQLState();
+    return $bit;
+}
+
+# Determines whether or not a group is active by checking 
+# the "isactive" column for the group in the "groups" table.
+# Note: This function selects groups by bit rather than by name.
+sub GroupIsActive {
+    my ($groupbit) = (@_);
+    $groupbit ||= 0;
+    ConnectToDatabase();
+    SendSQL("select isactive from groups where bit=$groupbit");
+    my $isactive = FetchOneColumn();
+    return $isactive;
+}
+
 # Determines if the given bug_status string represents an "Opened" bug.  This
 # routine ought to be paramaterizable somehow, as people tend to introduce
 # new states into Bugzilla.
@@ -987,36 +1185,87 @@ sub RemoveVotes {
     if ($who) {
         $whopart = " AND votes.who = $who";
     }
-    SendSQL("SELECT profiles.login_name, votes.count " .
-            "FROM votes, profiles " .
+    SendSQL("SELECT profiles.login_name, profiles.userid, votes.count, " .
+            "products.votesperuser, products.maxvotesperbug " .
+            "FROM profiles " . 
+            "LEFT JOIN votes ON profiles.userid = votes.who " .
+            "LEFT JOIN bugs USING(bug_id) " .
+            "LEFT JOIN products USING(product)" .
             "WHERE votes.bug_id = $id " .
-            "AND profiles.userid = votes.who" .
             $whopart);
     my @list;
     while (MoreSQLData()) {
-        my ($name, $count) = (FetchSQLData());
-        push(@list, [$name, $count]);
+        my ($name, $userid, $oldvotes, $votesperuser, $maxvotesperbug) = (FetchSQLData());
+        push(@list, [$name, $userid, $oldvotes, $votesperuser, $maxvotesperbug]);
     }
     if (0 < @list) {
         foreach my $ref (@list) {
-            my ($name, $count) = (@$ref);
+            my ($name, $userid, $oldvotes, $votesperuser, $maxvotesperbug) = (@$ref);
+            my $s;
+
+            $maxvotesperbug = $votesperuser if ($votesperuser < $maxvotesperbug);
+
+            # If this product allows voting and the user's votes are in
+            # the acceptable range, then don't do anything.
+            next if $votesperuser && $oldvotes <= $maxvotesperbug;
+
+            # If the user has more votes on this bug than this product
+            # allows, then reduce the number of votes so it fits
+            my $newvotes = $votesperuser ? $maxvotesperbug : 0;
+
+            my $removedvotes = $oldvotes - $newvotes;
+
+            $s = $oldvotes == 1 ? "" : "s";
+            my $oldvotestext = "You had $oldvotes vote$s on this bug.";
+
+            $s = $removedvotes == 1 ? "" : "s";
+            my $removedvotestext = "You had $removedvotes vote$s removed from this bug.";
+
+            my $newvotestext;
+            if ($newvotes) {
+                SendSQL("UPDATE votes SET count = $newvotes " .
+                        "WHERE bug_id = $id AND who = $userid");
+                $s = $newvotes == 1 ? "" : "s";
+                $newvotestext = "You still have $newvotes vote$s on this bug."
+            } else {
+                SendSQL("DELETE FROM votes WHERE bug_id = $id AND who = $userid");
+                $newvotestext = "You have no more votes remaining on this bug.";
+            }
+
+            # Notice that we did not make sure that the user fit within the $votesperuser
+            # range.  This is considered to be an acceptable alternative to losing votes
+            # during product moves.  Then next time the user attempts to change their votes,
+            # they will be forced to fit within the $votesperuser limit.
+
+            # Now lets send the e-mail to alert the user to the fact that their votes have
+            # been reduced or removed.
             my $sendmailparm = '-ODeliveryMode=deferred';
             if (Param('sendmailnow')) {
                $sendmailparm = '';
             }
             if (open(SENDMAIL, "|/usr/lib/sendmail $sendmailparm -t")) {
                 my %substs;
+
                 $substs{"to"} = $name;
                 $substs{"bugid"} = $id;
                 $substs{"reason"} = $reason;
-                $substs{"count"} = $count;
+
+                $substs{"votesremoved"} = $removedvotes;
+                $substs{"votesold"} = $oldvotes;
+                $substs{"votesnew"} = $newvotes;
+
+                $substs{"votesremovedtext"} = $removedvotestext;
+                $substs{"votesoldtext"} = $oldvotestext;
+                $substs{"votesnewtext"} = $newvotestext;
+
+                $substs{"count"} = $removedvotes . "\n    " . $newvotestext;
+
                 my $msg = PerformSubsts(Param("voteremovedmail"),
                                         \%substs);
                 print SENDMAIL $msg;
                 close SENDMAIL;
             }
         }
-        SendSQL("DELETE FROM votes WHERE bug_id = $id" . $whopart);
         SendSQL("SELECT SUM(count) FROM votes WHERE bug_id = $id");
         my $v = FetchOneColumn();
         $v ||= 0;
@@ -1064,6 +1313,35 @@ sub Param ($) {
     die "Can't find param named $value";
 }
 
+# Take two comma or space separated strings and return what
+# values were removed from or added to the new one.
+sub DiffStrings {
+    my ($oldstr, $newstr) = @_;
+
+    # Split the old and new strings into arrays containing their values.
+    $oldstr =~ s/[\s,]+/ /g;
+    $newstr =~ s/[\s,]+/ /g;
+    my @old = split(" ", $oldstr);
+    my @new = split(" ", $newstr);
+
+    my (@remove, @add) = ();
+
+    # Find values that were removed
+    foreach my $value (@old) {
+        push (@remove, $value) if !grep($_ eq $value, @new);
+    }
+
+    # Find values that were added
+    foreach my $value (@new) {
+        push (@add, $value) if !grep($_ eq $value, @old);
+    }
+
+    my $removed = join (", ", @remove);
+    my $added = join (", ", @add);
+
+    return ($removed, $added);
+}
+
 sub PerformSubsts {
     my ($str, $substs) = (@_);
     $str =~ s/%([a-z]*)%/(defined $substs->{$1} ? $substs->{$1} : Param($1))/eg;
